"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestRuleSetRevision = exports.getRuleSetRevisionFromJson = exports.deserializeRuleSetRevision = exports.serializeRuleSetRevision = exports.isRuleSetV2 = exports.isRuleSetV1 = void 0;
const bn_js_1 = require("bn.js");
const revisionMap_1 = require("./revisionMap");
const v1_1 = require("./v1");
const v2_1 = require("./v2");
const isRuleSetV1 = (ruleSet) => {
    return ruleSet.libVersion === 1;
};
exports.isRuleSetV1 = isRuleSetV1;
const isRuleSetV2 = (ruleSet) => {
    return ruleSet.libVersion === 2;
};
exports.isRuleSetV2 = isRuleSetV2;
const serializeRuleSetRevision = (ruleSet) => {
    if ((0, exports.isRuleSetV1)(ruleSet))
        return (0, v1_1.serializeRuleSetRevisionV1)(ruleSet);
    if ((0, exports.isRuleSetV2)(ruleSet))
        return (0, v2_1.serializeRuleSetRevisionV2)(ruleSet);
    throw new Error('Unknown rule set version: ' + ruleSet.libVersion);
};
exports.serializeRuleSetRevision = serializeRuleSetRevision;
const deserializeRuleSetRevision = (buffer, offset = 0) => {
    const libVersion = buffer[offset];
    if (libVersion === 1)
        return (0, v1_1.deserializeRuleSetRevisionV1)(buffer, offset);
    if (libVersion === 2)
        return (0, v2_1.deserializeRuleSetRevisionV2)(buffer, offset);
    throw new Error('Unknown rule set version: ' + libVersion);
};
exports.deserializeRuleSetRevision = deserializeRuleSetRevision;
const getRuleSetRevisionFromJson = (json) => {
    const ruleSet = JSON.parse(json);
    if ((0, exports.isRuleSetV1)(ruleSet) || (0, exports.isRuleSetV2)(ruleSet))
        return ruleSet;
    throw new Error('Unknown rule set version: ' + ruleSet.libVersion);
};
exports.getRuleSetRevisionFromJson = getRuleSetRevisionFromJson;
const getLatestRuleSetRevision = (data) => {
    const header = (0, revisionMap_1.getHeader)(data);
    const revmap = (0, revisionMap_1.getRevisionMapV1)(data);
    const latestRevision = new bn_js_1.BN(revmap.ruleSetRevisions[revmap.ruleSetRevisions.length - 1]).toNumber();
    const endOfRuleSet = new bn_js_1.BN(header.revMapVersionLocation).toNumber();
    return (0, exports.deserializeRuleSetRevision)(data.subarray(latestRevision, endOfRuleSet));
};
exports.getLatestRuleSetRevision = getLatestRuleSetRevision;
//# sourceMappingURL=revision.js.map