"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRuleV2FromV1 = exports.getRuleSetRevisionV2FromV1 = exports.deserializeRuleSetRevisionV2 = exports.serializeRuleSetRevisionV2 = void 0;
const beet = __importStar(require("@metaplex-foundation/beet"));
const web3_js_1 = require("@solana/web3.js");
const additionalSigner_1 = require("./additionalSigner");
const all_1 = require("./all");
const amount_1 = require("./amount");
const any_1 = require("./any");
const base58PublicKey_1 = require("./base58PublicKey");
const helpers_1 = require("./helpers");
const namespace_1 = require("./namespace");
const not_1 = require("./not");
const pass_1 = require("./pass");
const pdaMatch_1 = require("./pdaMatch");
const programOwned_1 = require("./programOwned");
const programOwnedList_1 = require("./programOwnedList");
const programOwnedTree_1 = require("./programOwnedTree");
const pubkeyListMatch_1 = require("./pubkeyListMatch");
const pubkeyMatch_1 = require("./pubkeyMatch");
const pubkeyTreeMatch_1 = require("./pubkeyTreeMatch");
const rule_1 = require("./rule");
const serializeRuleSetRevisionV2 = (ruleSet) => {
    const tuples = Object.entries(ruleSet.operations);
    const operations = tuples.map(([operation]) => operation);
    const rules = tuples.map(([, rule]) => rule);
    const ruleSize = operations.length;
    const headerBuffer = Buffer.alloc(8);
    beet.u32.write(headerBuffer, 0, 2);
    beet.u32.write(headerBuffer, 4, ruleSize);
    const ownerBuffer = (0, helpers_1.serializePublicKey)(ruleSet.owner);
    const nameBuffer = Buffer.alloc(32);
    nameBuffer.write(ruleSet.name);
    const operationsBuffer = Buffer.concat(operations.map((operation) => {
        const buffer = Buffer.alloc(32);
        buffer.write(operation);
        return buffer;
    }));
    const rulesBuffer = (0, rule_1.serializeRulesV2)(rules);
    return Buffer.concat([headerBuffer, ownerBuffer, nameBuffer, operationsBuffer, rulesBuffer]);
};
exports.serializeRuleSetRevisionV2 = serializeRuleSetRevisionV2;
const deserializeRuleSetRevisionV2 = (buffer, offset = 0) => {
    const libVersion = beet.u32.read(buffer, offset);
    offset += 4;
    if (libVersion !== 2) {
        throw new Error('Expected a rule set version 2, got version ' + libVersion);
    }
    const ruleSize = beet.u32.read(buffer, offset);
    offset += 4;
    const owner = (0, helpers_1.deserializePublicKey)(buffer, offset);
    offset += 32;
    const name = buffer
        .subarray(offset, offset + 32)
        .toString()
        .replace(/\u0000/g, '');
    offset += 32;
    const operations = [];
    for (let index = 0; index < ruleSize; index++) {
        operations.push(buffer
            .subarray(offset, offset + 32)
            .toString()
            .replace(/\u0000/g, ''));
        offset += 32;
    }
    const rules = (0, rule_1.deserializeRulesV2)(buffer, ruleSize, offset);
    const tuples = operations.map((operation, index) => [
        operation,
        rules[index],
    ]);
    return { libVersion: 2, name, owner, operations: Object.fromEntries(tuples) };
};
exports.deserializeRuleSetRevisionV2 = deserializeRuleSetRevisionV2;
const getRuleSetRevisionV2FromV1 = (ruleSetV1) => {
    return {
        libVersion: 2,
        name: ruleSetV1.ruleSetName,
        owner: (0, base58PublicKey_1.toBase58PublicKey)(new web3_js_1.PublicKey(ruleSetV1.owner)),
        operations: Object.fromEntries(Object.entries(ruleSetV1.operations).map(([operation, rule]) => [
            operation,
            (0, exports.getRuleV2FromV1)(rule),
        ])),
    };
};
exports.getRuleSetRevisionV2FromV1 = getRuleSetRevisionV2FromV1;
const getRuleV2FromV1 = (ruleV1) => {
    if (ruleV1 === 'Namespace') {
        return (0, namespace_1.namespaceV2)();
    }
    if (ruleV1 === 'Pass') {
        return (0, pass_1.passV2)();
    }
    if ('AdditionalSigner' in ruleV1) {
        return (0, additionalSigner_1.additionalSignerV2)(new web3_js_1.PublicKey(ruleV1.AdditionalSigner.account));
    }
    if ('All' in ruleV1) {
        return (0, all_1.allV2)(ruleV1.All.rules.map(exports.getRuleV2FromV1));
    }
    if ('Amount' in ruleV1) {
        return (0, amount_1.amountV2)(ruleV1.Amount.field, ruleV1.Amount.operator, ruleV1.Amount.amount);
    }
    if ('Any' in ruleV1) {
        return (0, any_1.anyV2)(ruleV1.Any.rules.map(exports.getRuleV2FromV1));
    }
    if ('Not' in ruleV1) {
        return (0, not_1.notV2)((0, exports.getRuleV2FromV1)(ruleV1.Not.rule));
    }
    if ('PDAMatch' in ruleV1) {
        return (0, pdaMatch_1.pdaMatchV2)(ruleV1.PDAMatch.pda_field, new web3_js_1.PublicKey(ruleV1.PDAMatch.program), ruleV1.PDAMatch.seeds_field);
    }
    if ('ProgramOwned' in ruleV1) {
        return (0, programOwned_1.programOwnedV2)(ruleV1.ProgramOwned.field, new web3_js_1.PublicKey(ruleV1.ProgramOwned.program));
    }
    if ('ProgramOwnedList' in ruleV1) {
        return (0, programOwnedList_1.programOwnedListV2)(ruleV1.ProgramOwnedList.field, ruleV1.ProgramOwnedList.programs.map((p) => new web3_js_1.PublicKey(p)));
    }
    if ('ProgramOwnedTree' in ruleV1) {
        return (0, programOwnedTree_1.programOwnedTreeV2)(ruleV1.ProgramOwnedTree.pubkey_field, ruleV1.ProgramOwnedTree.proof_field, new Uint8Array(ruleV1.ProgramOwnedTree.root));
    }
    if ('PubkeyListMatch' in ruleV1) {
        return (0, pubkeyListMatch_1.pubkeyListMatchV2)(ruleV1.PubkeyListMatch.field, ruleV1.PubkeyListMatch.pubkeys.map((p) => new web3_js_1.PublicKey(p)));
    }
    if ('PubkeyMatch' in ruleV1) {
        return (0, pubkeyMatch_1.pubkeyMatchV2)(ruleV1.PubkeyMatch.field, new web3_js_1.PublicKey(ruleV1.PubkeyMatch.pubkey));
    }
    if ('PubkeyTreeMatch' in ruleV1) {
        return (0, pubkeyTreeMatch_1.pubkeyTreeMatchV2)(ruleV1.PubkeyTreeMatch.pubkey_field, ruleV1.PubkeyTreeMatch.proof_field, new Uint8Array(ruleV1.PubkeyTreeMatch.root));
    }
    throw new Error('Unknown rule: ' + JSON.stringify(ruleV1));
};
exports.getRuleV2FromV1 = getRuleV2FromV1;
//# sourceMappingURL=revision.js.map